@@ -8,7 +8,7 @@
  *
  * 1. Copyright Notice
  *
- * Some or all of this work - Copyright (c) 1999 - 2018, Intel Corp.
+ * Some or all of this work - Copyright (c) 1999 - 2021, Intel Corp.
  * All rights reserved.
  *
  * 2. License
@@ -202,7 +202,7 @@ DtCompileAsf (
     {
         SubtableStart = *PFieldList;
         Status = DtCompileTable (PFieldList, AcpiDmTableInfoAsfHdr,
-            &Subtable, TRUE);
+            &Subtable);
         if (ACPI_FAILURE (Status))
         {
             return (Status);
@@ -247,7 +247,7 @@ DtCompileAsf (
             return (AE_ERROR);
         }
 
-        Status = DtCompileTable (PFieldList, InfoTable, &Subtable, TRUE);
+        Status = DtCompileTable (PFieldList, InfoTable, &Subtable);
         if (ACPI_FAILURE (Status))
         {
             return (Status);
@@ -307,7 +307,7 @@ DtCompileAsf (
                 while (DataCount > 0)
                 {
                     Status = DtCompileTable (PFieldList, DataInfoTable,
-                        &Subtable, TRUE);
+                        &Subtable);
                     if (ACPI_FAILURE (Status))
                     {
                         return (Status);
@@ -323,7 +323,7 @@ DtCompileAsf (
                 for (i = 0; i < DataCount; i++)
                 {
                     Status = DtCompileTable (PFieldList, DataInfoTable,
-                        &Subtable, TRUE);
+                        &Subtable);
                     if (ACPI_FAILURE (Status))
                     {
                         return (Status);
@@ -399,7 +399,7 @@ DtCompileCsrt (
         /* Resource group subtable */
 
         Status = DtCompileTable (PFieldList, AcpiDmTableInfoCsrt0,
-            &Subtable, TRUE);
+            &Subtable);
         if (ACPI_FAILURE (Status))
         {
             return (Status);
@@ -424,7 +424,7 @@ DtCompileCsrt (
         /* Shared info subtable (One per resource group) */
 
         Status = DtCompileTable (PFieldList, AcpiDmTableInfoCsrt1,
-            &Subtable, TRUE);
+            &Subtable);
         if (ACPI_FAILURE (Status))
         {
             return (Status);
@@ -438,7 +438,7 @@ DtCompileCsrt (
         {
 
             Status = DtCompileTable (PFieldList, AcpiDmTableInfoCsrt2,
-                &Subtable, TRUE);
+                &Subtable);
             if (ACPI_FAILURE (Status))
             {
                 return (Status);
@@ -451,7 +451,7 @@ DtCompileCsrt (
             if (*PFieldList)
             {
                 Status = DtCompileTable (PFieldList, AcpiDmTableInfoCsrt2a,
-                    &Subtable, TRUE);
+                    &Subtable);
                 if (ACPI_FAILURE (Status))
                 {
                     return (Status);
@@ -504,7 +504,7 @@ DtCompileDbg2 (
 
     /* Main table */
 
-    Status = DtCompileTable (PFieldList, AcpiDmTableInfoDbg2, &Subtable, TRUE);
+    Status = DtCompileTable (PFieldList, AcpiDmTableInfoDbg2, &Subtable);
     if (ACPI_FAILURE (Status))
     {
         return (Status);
@@ -529,7 +529,7 @@ DtCompileDbg2 (
         /* Subtable: Debug Device Information */
 
         Status = DtCompileTable (PFieldList, AcpiDmTableInfoDbg2Device,
-            &Subtable, TRUE);
+            &Subtable);
         if (ACPI_FAILURE (Status))
         {
             return (Status);
@@ -550,7 +550,7 @@ DtCompileDbg2 (
         for (i = 0; *PFieldList && (i < DeviceInfo->RegisterCount); i++)
         {
             Status = DtCompileTable (PFieldList, AcpiDmTableInfoDbg2Addr,
-                &Subtable, TRUE);
+                &Subtable);
             if (ACPI_FAILURE (Status))
             {
                 return (Status);
@@ -566,7 +566,7 @@ DtCompileDbg2 (
         for (i = 0; *PFieldList && (i < DeviceInfo->RegisterCount); i++)
         {
             Status = DtCompileTable (PFieldList, AcpiDmTableInfoDbg2Size,
-                &Subtable, TRUE);
+                &Subtable);
             if (ACPI_FAILURE (Status))
             {
                 return (Status);
@@ -580,7 +580,7 @@ DtCompileDbg2 (
 
         DeviceInfo->NamepathOffset = CurrentOffset;
         Status = DtCompileTable (PFieldList, AcpiDmTableInfoDbg2Name,
-            &Subtable, TRUE);
+            &Subtable);
         if (ACPI_FAILURE (Status))
         {
             return (Status);
@@ -595,7 +595,7 @@ DtCompileDbg2 (
         /* OemData - Variable-length data (Optional, size = OemDataLength) */
 
         Status = DtCompileTable (PFieldList, AcpiDmTableInfoDbg2OemData,
-            &Subtable, TRUE);
+            &Subtable);
         if (Status == AE_END_OF_TABLE)
         {
             /* optional field was not found and we're at the end of the file */
@@ -659,7 +659,7 @@ DtCompileDmar (
     UINT32                  PciPathLength;
 
 
-    Status = DtCompileTable (PFieldList, AcpiDmTableInfoDmar, &Subtable, TRUE);
+    Status = DtCompileTable (PFieldList, AcpiDmTableInfoDmar, &Subtable);
     if (ACPI_FAILURE (Status))
     {
         return (Status);
@@ -675,7 +675,7 @@ DtCompileDmar (
 
         SubtableStart = *PFieldList;
         Status = DtCompileTable (PFieldList, AcpiDmTableInfoDmarHdr,
-            &Subtable, TRUE);
+            &Subtable);
         if (ACPI_FAILURE (Status))
         {
             return (Status);
@@ -722,7 +722,7 @@ DtCompileDmar (
 
         /* DMAR Subtable */
 
-        Status = DtCompileTable (PFieldList, InfoTable, &Subtable, TRUE);
+        Status = DtCompileTable (PFieldList, InfoTable, &Subtable);
         if (ACPI_FAILURE (Status))
         {
             return (Status);
@@ -749,7 +749,7 @@ DtCompileDmar (
         while (DeviceScopeLength)
         {
             Status = DtCompileTable (PFieldList, AcpiDmTableInfoDmarScope,
-                &Subtable, FALSE);
+                &Subtable);
             if (Status == AE_NOT_FOUND)
             {
                 break;
@@ -767,7 +767,7 @@ DtCompileDmar (
             while (PciPathLength)
             {
                 Status = DtCompileTable (PFieldList, TableInfoDmarPciPath,
-                    &Subtable, FALSE);
+                    &Subtable);
                 if (Status == AE_NOT_FOUND)
                 {
                     DtPopSubtable ();
@@ -823,7 +823,7 @@ DtCompileDrtm (
     /* Compile DRTM header */
 
     Status = DtCompileTable (PFieldList, AcpiDmTableInfoDrtm,
-        &Subtable, TRUE);
+        &Subtable);
     if (ACPI_FAILURE (Status))
     {
         return (Status);
@@ -831,7 +831,7 @@ DtCompileDrtm (
     DtInsertSubtable (ParentTable, Subtable);
 
     /*
-     * Using ACPI_SUB_PTR, We needn't define a seperate structure. Care
+     * Using ACPI_SUB_PTR, We needn't define a separate structure. Care
      * should be taken to avoid accessing ACPI_TABLE_HADER fields.
      */
 #if 0
@@ -841,7 +841,7 @@ DtCompileDrtm (
     /* Compile VTL */
 
     Status = DtCompileTable (PFieldList, AcpiDmTableInfoDrtm0,
-        &Subtable, TRUE);
+        &Subtable);
     if (ACPI_FAILURE (Status))
     {
         return (Status);
@@ -857,7 +857,7 @@ DtCompileDrtm (
     while (*PFieldList)
     {
         Status = DtCompileTable (PFieldList, AcpiDmTableInfoDrtm0a,
-            &Subtable, TRUE);
+            &Subtable);
         if (ACPI_FAILURE (Status))
         {
             return (Status);
@@ -877,7 +877,7 @@ DtCompileDrtm (
     /* Compile RL */
 
     Status = DtCompileTable (PFieldList, AcpiDmTableInfoDrtm1,
-        &Subtable, TRUE);
+        &Subtable);
     if (ACPI_FAILURE (Status))
     {
         return (Status);
@@ -893,7 +893,7 @@ DtCompileDrtm (
     while (*PFieldList)
     {
         Status = DtCompileTable (PFieldList, AcpiDmTableInfoDrtm1a,
-            &Subtable, TRUE);
+            &Subtable);
         if (ACPI_FAILURE (Status))
         {
             return (Status);
@@ -915,7 +915,7 @@ DtCompileDrtm (
     /* Compile DPS */
 
     Status = DtCompileTable (PFieldList, AcpiDmTableInfoDrtm2,
-        &Subtable, TRUE);
+        &Subtable);
     if (ACPI_FAILURE (Status))
     {
         return (Status);
@@ -1002,15 +1002,37 @@ DtCompileGtdt (
     ACPI_SUBTABLE_HEADER    *GtdtHeader;
     ACPI_DMTABLE_INFO       *InfoTable;
     UINT32                  GtCount;
+    ACPI_TABLE_HEADER       *Header;
 
 
+    ParentTable = DtPeekSubtable ();
+
+    Header = ACPI_CAST_PTR (ACPI_TABLE_HEADER, ParentTable->Buffer);
+
+    /* Compile the main table */
+
     Status = DtCompileTable (PFieldList, AcpiDmTableInfoGtdt,
-        &Subtable, TRUE);
+        &Subtable);
     if (ACPI_FAILURE (Status))
     {
         return (Status);
     }
 
+    /* GTDT revision 3 later contains 2 extra fields before subtables */
+
+    if (Header->Revision > 2)
+    {
+        ParentTable = DtPeekSubtable ();
+        DtInsertSubtable (ParentTable, Subtable);
+
+        Status = DtCompileTable (PFieldList,
+            AcpiDmTableInfoGtdtEl2, &Subtable);
+        if (ACPI_FAILURE (Status))
+        {
+            return (Status);
+        }
+    }
+
     ParentTable = DtPeekSubtable ();
     DtInsertSubtable (ParentTable, Subtable);
 
@@ -1018,7 +1040,7 @@ DtCompileGtdt (
     {
         SubtableStart = *PFieldList;
         Status = DtCompileTable (PFieldList, AcpiDmTableInfoGtdtHdr,
-            &Subtable, TRUE);
+            &Subtable);
         if (ACPI_FAILURE (Status))
         {
             return (Status);
@@ -1048,7 +1070,7 @@ DtCompileGtdt (
             return (AE_ERROR);
         }
 
-        Status = DtCompileTable (PFieldList, InfoTable, &Subtable, TRUE);
+        Status = DtCompileTable (PFieldList, InfoTable, &Subtable);
         if (ACPI_FAILURE (Status))
         {
             return (Status);
@@ -1074,7 +1096,7 @@ DtCompileGtdt (
             while (GtCount)
             {
                 Status = DtCompileTable (PFieldList, AcpiDmTableInfoGtdt0a,
-                    &Subtable, TRUE);
+                    &Subtable);
                 if (ACPI_FAILURE (Status))
                 {
                     return (Status);
@@ -1128,7 +1150,7 @@ DtCompileFpdt (
     {
         SubtableStart = *PFieldList;
         Status = DtCompileTable (PFieldList, AcpiDmTableInfoFpdtHdr,
-            &Subtable, TRUE);
+            &Subtable);
         if (ACPI_FAILURE (Status))
         {
             return (Status);
@@ -1159,7 +1181,7 @@ DtCompileFpdt (
             break;
         }
 
-        Status = DtCompileTable (PFieldList, InfoTable, &Subtable, TRUE);
+        Status = DtCompileTable (PFieldList, InfoTable, &Subtable);
         if (ACPI_FAILURE (Status))
         {
             return (Status);
@@ -1201,7 +1223,7 @@ DtCompileHest (
 
 
     Status = DtCompileTable (PFieldList, AcpiDmTableInfoHest,
-        &Subtable, TRUE);
+        &Subtable);
     if (ACPI_FAILURE (Status))
     {
         return (Status);
@@ -1272,7 +1294,7 @@ DtCompileHest (
             return (AE_ERROR);
         }
 
-        Status = DtCompileTable (PFieldList, InfoTable, &Subtable, TRUE);
+        Status = DtCompileTable (PFieldList, InfoTable, &Subtable);
         if (ACPI_FAILURE (Status))
         {
             return (Status);
@@ -1312,7 +1334,7 @@ DtCompileHest (
         while (BankCount)
         {
             Status = DtCompileTable (PFieldList, AcpiDmTableInfoHestBank,
-                &Subtable, TRUE);
+                &Subtable);
             if (ACPI_FAILURE (Status))
             {
                 return (Status);
@@ -1362,7 +1384,7 @@ DtCompileHmat (
     ParentTable = DtPeekSubtable ();
 
     Status = DtCompileTable (PFieldList, AcpiDmTableInfoHmat,
-        &Subtable, TRUE);
+        &Subtable);
     if (ACPI_FAILURE (Status))
     {
         return (Status);
@@ -1375,7 +1397,7 @@ DtCompileHmat (
 
         SubtableStart = *PFieldList;
         Status = DtCompileTable (PFieldList, AcpiDmTableInfoHmatHdr,
-            &Subtable, TRUE);
+            &Subtable);
         if (ACPI_FAILURE (Status))
         {
             return (Status);
@@ -1410,7 +1432,7 @@ DtCompileHmat (
             return (AE_ERROR);
         }
 
-        Status = DtCompileTable (PFieldList, InfoTable, &Subtable, TRUE);
+        Status = DtCompileTable (PFieldList, InfoTable, &Subtable);
         if (ACPI_FAILURE (Status))
         {
             return (Status);
@@ -1433,7 +1455,7 @@ DtCompileHmat (
             while (*PFieldList)
             {
                 Status = DtCompileTable (PFieldList,
-                    AcpiDmTableInfoHmat1a, &Subtable, TRUE);
+                    AcpiDmTableInfoHmat1a, &Subtable);
                 if (ACPI_FAILURE (Status))
                 {
                     return (Status);
@@ -1454,7 +1476,7 @@ DtCompileHmat (
             while (*PFieldList)
             {
                 Status = DtCompileTable (PFieldList,
-                    AcpiDmTableInfoHmat1b, &Subtable, TRUE);
+                    AcpiDmTableInfoHmat1b, &Subtable);
                 if (ACPI_FAILURE (Status))
                 {
                     return (Status);
@@ -1479,7 +1501,7 @@ DtCompileHmat (
             while (*PFieldList)
             {
                 Status = DtCompileTable (PFieldList,
-                    AcpiDmTableInfoHmat1c, &Subtable, TRUE);
+                    AcpiDmTableInfoHmat1c, &Subtable);
                 if (ACPI_FAILURE (Status))
                 {
                     return (Status);
@@ -1513,7 +1535,7 @@ DtCompileHmat (
             while (*PFieldList)
             {
                 Status = DtCompileTable (PFieldList,
-                    AcpiDmTableInfoHmat2a, &Subtable, TRUE);
+                    AcpiDmTableInfoHmat2a, &Subtable);
                 if (ACPI_FAILURE (Status))
                 {
                     return (Status);
@@ -1576,7 +1598,7 @@ DtCompileIort (
     ParentTable = DtPeekSubtable ();
 
     Status = DtCompileTable (PFieldList, AcpiDmTableInfoIort,
-        &Subtable, TRUE);
+        &Subtable);
     if (ACPI_FAILURE (Status))
     {
         return (Status);
@@ -1598,7 +1620,7 @@ DtCompileIort (
      */
     Iort->NodeOffset = sizeof (ACPI_TABLE_IORT);
     Status = DtCompileTable (PFieldList, AcpiDmTableInfoIortPad,
-        &Subtable, TRUE);
+        &Subtable);
     if (ACPI_FAILURE (Status))
     {
         return (Status);
@@ -1624,7 +1646,7 @@ DtCompileIort (
     {
         SubtableStart = *PFieldList;
         Status = DtCompileTable (PFieldList, AcpiDmTableInfoIortHdr,
-            &Subtable, TRUE);
+            &Subtable);
         if (ACPI_FAILURE (Status))
         {
             return (Status);
@@ -1642,7 +1664,7 @@ DtCompileIort (
         case ACPI_IORT_NODE_ITS_GROUP:
 
             Status = DtCompileTable (PFieldList, AcpiDmTableInfoIort0,
-                &Subtable, TRUE);
+                &Subtable);
             if (ACPI_FAILURE (Status))
             {
                 return (Status);
@@ -1656,7 +1678,7 @@ DtCompileIort (
             while (*PFieldList)
             {
                 Status = DtCompileTable (PFieldList, AcpiDmTableInfoIort0a,
-                    &Subtable, TRUE);
+                    &Subtable);
                 if (ACPI_FAILURE (Status))
                 {
                     return (Status);
@@ -1677,7 +1699,7 @@ DtCompileIort (
         case ACPI_IORT_NODE_NAMED_COMPONENT:
 
             Status = DtCompileTable (PFieldList, AcpiDmTableInfoIort1,
-                &Subtable, TRUE);
+                &Subtable);
             if (ACPI_FAILURE (Status))
             {
                 return (Status);
@@ -1692,7 +1714,7 @@ DtCompileIort (
              * for filling this field.
              */
             Status = DtCompileTable (PFieldList, AcpiDmTableInfoIort1a,
-                &Subtable, TRUE);
+                &Subtable);
             if (ACPI_FAILURE (Status))
             {
                 return (Status);
@@ -1729,7 +1751,7 @@ DtCompileIort (
         case ACPI_IORT_NODE_PCI_ROOT_COMPLEX:
 
             Status = DtCompileTable (PFieldList, AcpiDmTableInfoIort2,
-                &Subtable, TRUE);
+                &Subtable);
             if (ACPI_FAILURE (Status))
             {
                 return (Status);
@@ -1742,7 +1764,7 @@ DtCompileIort (
         case ACPI_IORT_NODE_SMMU:
 
             Status = DtCompileTable (PFieldList, AcpiDmTableInfoIort3,
-                &Subtable, TRUE);
+                &Subtable);
             if (ACPI_FAILURE (Status))
             {
                 return (Status);
@@ -1756,7 +1778,7 @@ DtCompileIort (
 
             IortSmmu->GlobalInterruptOffset = NodeLength;
             Status = DtCompileTable (PFieldList, AcpiDmTableInfoIort3a,
-                &Subtable, TRUE);
+                &Subtable);
             if (ACPI_FAILURE (Status))
             {
                 return (Status);
@@ -1772,7 +1794,7 @@ DtCompileIort (
             while (*PFieldList)
             {
                 Status = DtCompileTable (PFieldList, AcpiDmTableInfoIort3b,
-                    &Subtable, TRUE);
+                    &Subtable);
                 if (ACPI_FAILURE (Status))
                 {
                     return (Status);
@@ -1797,7 +1819,7 @@ DtCompileIort (
             while (*PFieldList)
             {
                 Status = DtCompileTable (PFieldList, AcpiDmTableInfoIort3c,
-                    &Subtable, TRUE);
+                    &Subtable);
                 if (ACPI_FAILURE (Status))
                 {
                     return (Status);
@@ -1819,7 +1841,20 @@ DtCompileIort (
         case ACPI_IORT_NODE_SMMU_V3:
 
             Status = DtCompileTable (PFieldList, AcpiDmTableInfoIort4,
-                &Subtable, TRUE);
+                &Subtable);
+            if (ACPI_FAILURE (Status))
+            {
+                return (Status);
+            }
+
+            DtInsertSubtable (ParentTable, Subtable);
+            NodeLength += Subtable->Length;
+            break;
+
+        case ACPI_IORT_NODE_PMCG:
+
+            Status = DtCompileTable (PFieldList, AcpiDmTableInfoIort5,
+                &Subtable);
             if (ACPI_FAILURE (Status))
             {
                 return (Status);
@@ -1842,7 +1877,7 @@ DtCompileIort (
         while (*PFieldList)
         {
             Status = DtCompileTable (PFieldList, AcpiDmTableInfoIortMap,
-                &Subtable, TRUE);
+                &Subtable);
             if (ACPI_FAILURE (Status))
             {
                 return (Status);
@@ -1905,7 +1940,7 @@ DtCompileIvrs (
 
 
     Status = DtCompileTable (PFieldList, AcpiDmTableInfoIvrs,
-        &Subtable, TRUE);
+        &Subtable);
     if (ACPI_FAILURE (Status))
     {
         return (Status);
@@ -1918,7 +1953,7 @@ DtCompileIvrs (
     {
         SubtableStart = *PFieldList;
         Status = DtCompileTable (PFieldList, AcpiDmTableInfoIvrsHdr,
-            &Subtable, TRUE);
+            &Subtable);
         if (ACPI_FAILURE (Status))
         {
             return (Status);
@@ -1932,11 +1967,16 @@ DtCompileIvrs (
 
         switch (IvrsHeader->Type)
         {
-        case ACPI_IVRS_TYPE_HARDWARE:
+        case ACPI_IVRS_TYPE_HARDWARE1:
 
             InfoTable = AcpiDmTableInfoIvrs0;
             break;
 
+        case ACPI_IVRS_TYPE_HARDWARE2:
+
+            InfoTable = AcpiDmTableInfoIvrs01;
+            break;
+
         case ACPI_IVRS_TYPE_MEMORY1:
         case ACPI_IVRS_TYPE_MEMORY2:
         case ACPI_IVRS_TYPE_MEMORY3:
@@ -1950,7 +1990,7 @@ DtCompileIvrs (
             return (AE_ERROR);
         }
 
-        Status = DtCompileTable (PFieldList, InfoTable, &Subtable, TRUE);
+        Status = DtCompileTable (PFieldList, InfoTable, &Subtable);
         if (ACPI_FAILURE (Status))
         {
             return (Status);
@@ -1959,7 +1999,8 @@ DtCompileIvrs (
         ParentTable = DtPeekSubtable ();
         DtInsertSubtable (ParentTable, Subtable);
 
-        if (IvrsHeader->Type == ACPI_IVRS_TYPE_HARDWARE)
+        if (IvrsHeader->Type == ACPI_IVRS_TYPE_HARDWARE1 ||
+            IvrsHeader->Type == ACPI_IVRS_TYPE_HARDWARE2)
         {
             while (*PFieldList &&
                 !strcmp ((*PFieldList)->Name, "Entry Type"))
@@ -2012,7 +2053,7 @@ DtCompileIvrs (
                 }
 
                 Status = DtCompileTable (PFieldList, InfoTable,
-                    &Subtable, TRUE);
+                    &Subtable);
                 if (ACPI_FAILURE (Status))
                 {
                     return (Status);
